<?php

namespace app\admin\controller\order;

use app\admin\validate\order\OrderValidate;
use app\controller\Curd;
use app\model\Goods;
use app\model\Order;
use app\model\OrderSheet;
use support\Request;
use support\Response;

class DishesController extends Curd{
    public function __construct()
    {
        $this->model = new Order();
        $this->validate = true;
        $this->validateClass = new OrderValidate();
    }

    /**
     * @Desc 列表
     * @Author Gorden
     * @Date 2024/3/28 15:01
     *
     * @param Request $request
     * @return Response
     * @throws \support\exception\BusinessException
     */
    public function select(Request $request): Response
    {
        [$where, $format, $limit, $field, $order] = $this->selectInput($request);
        if (!empty($where['order_addtimes'])) {
            $where['order_addtimes'][0] = strtotime($where['order_addtimes'][0]);
            $where['order_addtimes'][1] = strtotime($where['order_addtimes'][1]);
        }

        $order = $request->get('order', 'desc');
        $field = $field ?? 'order_addtimes';
        // $where['order_category'] = 'DISHES';
        if (!empty($where['order_status_system']) && in_array($where['order_status_system'], ['PENDING', 'WAITING', 'SENDING', 'RECVING', 'SIGNED', 'CONFIRM'])) {
            $where['order_is_complete'] = 'N';
            $where['order_category'] = ['<>','RETURN'];
        }

        // if (!empty($where['order_status_system']) && $where['order_status_system'] == 'RETURN') {
        //     unset($where['order_status_system']);
        //     unset($where['order_is_complete']);
        //     $where['order_category'] = 'RETURN';
        // }
        // if ($request->get('return')){
        //     $where['order_return.order_return_status'] = $request->get('return');
        // }
        // if ($request->get('goods_classify') || $request->get('goods_name')){
            $where['order_id'] = '';
            $goodsClassify = 'MEALS';
            $goodsName = $request->get('goods_name','');
            if (!empty($goodsName) && empty($goodsClassify)){
                $goodsIds = Goods::where('goods_name','like','%'.$request->get('goods_name').'%')->pluck('goods_id')->toArray();
            }else if(!empty($goodsClassify)){
                if($goodsClassify == 'SERVICE'){
                    $goodsModel = Goods::whereIn('goods_classify',['SERVICE','CHNMED','CHNNCD']);
                }else{
                    $goodsModel = Goods::where('goods_classify',$goodsClassify);
                }

                if(!empty($goodsName)){
                    $goodsModel->where('goods_name','like','%'.$goodsName.'%');
                }

                $goodsIds = $goodsModel->pluck('goods_id')->toArray();
            }
            if($goodsIds){
                $orderIds = OrderSheet::whereIn('join_sheet_goods_id',$goodsIds)->pluck('join_sheet_order_id')->toArray();
                if($orderIds){
                    $where['order_id'] = ['in',implode(',',$orderIds)];
                }
            }
        // }

        $query = $this->doSelect($where, $field, $order);
        return $this->doFormat($query, $format, $limit);
    }

    protected function doSelect(array $where, string $field = null, string $order = 'desc')
    {
        $model = $this->model->with([
            'sheets' => function ($query) {
                $query->select('join_sheet_order_id', 'order_sheet_id', 'join_sheet_goods_id', 'order_sheet_num','order_sheet_price');
            },
            'member' => function ($query) {
                $query->select('member_id', 'member_mobile');
            },
            'cert' => function ($query) {
                $query->select('join_cert_member_id', 'member_cert_name');
            },
//            'return' => function ($query) use ($where){
//                if (isset($where['return'])){
//                    dump($where['return']);
//                    $query = $query->where('order_return_status',$where['return']);
//                }
//                $query->select('orders_return_id', 'join_return_order_id', 'order_return_status');
//            },
            'express' => function ($query) {
                $query->select('join_express_order_id', 'order_express_type');
            }
        ])->leftJoin('order_return','order_return.join_return_order_id','=','order.order_id');
        // ->leftJoin('order_sheet','join_sheet_order_id','=','order.order_id');
        foreach ($where as $column => $value) {
            if (is_array($value)) {
                if ($value[0] === 'like' || $value[0] === 'not like') {
                    $model = $model->where($column, $value[0], "%$value[1]%");
                } elseif (in_array($value[0], ['>', '=', '<', '<>'])) {
                    $model = $model->where($column, $value[0], $value[1]);
                } elseif ($value[0] == 'in' && !empty($value[1])) {
                    $valArr = $value[1];
                    if (is_string($value[1])) {
                        $valArr = explode(",", trim($value[1]));
                    }
                    $model = $model->whereIn($column, $valArr);
                } elseif ($value[0] == 'not in' && !empty($value[1])) {
                    $valArr = $value[1];
                    if (is_string($value[1])) {
                        $valArr = explode(",", trim($value[1]));
                    }
                    $model = $model->whereNotIn($column, $valArr);
                } elseif ($value[0] == 'null') {
                    $model = $model->whereNull($column);
                } elseif ($value[0] == 'not null') {
                    $model = $model->whereNotNull($column);
                } elseif ($value[0] !== '' || $value[1] !== '') {
                    $model = $model->whereBetween($column, $value);
                }
            } else {
                $model = $model->where($column, $value);
            }
        }
        if ($field) {
            $model = $model->orderBy($field, $order);
        }
        $model = $model->select('order.*','order_return.orders_return_id', 'order_return.join_return_order_id', 'order_return.order_return_status','order_return.order_return_apply_json','order_return.order_return_remark');
        return $model;
    }

    public function afterQuery($items)
    {
        foreach ($items as &$item) {
            $sheetDeng = '';
            $item['sheet'] = $item['sheets'][0] ?? [];
            if (!empty($item['sheet'])) {
                $goods = Goods::where('goods_id', $item['sheet']['join_sheet_goods_id'])->first();
                if(count($item['sheets']) > 1 && $goods->goods_classify == 'MEALS'){
                    $sheetDeng = ' 等餐品';
                }
                $item['sheet']['goods_name'] = ($goods && $goods->goods_name) ? $goods->goods_name.$sheetDeng : '';
                $item['sheet']['goods_classify'] = $goods->goods_classify ?? '';
                $item['sheet']['order_sheet_num'] = intval($item['sheet']['order_sheet_num']);
            }
            unset($item['sheets']);
            if (isset($item['orders_return_id'])){
                $item['return'] = [
                    'orders_return_id'=>$item['orders_return_id'],
                    'join_return_order_id' => $item['join_return_order_id'],
                    'order_return_status' => $item['order_return_status'],
                    'order_return_apply_json' => $item['order_return_apply_json'],
                    'order_return_remark'=>$item['order_return_remark']
                ];
            }
        }

        return $items;
    }
}